import inspect
import logging
import os

from discord.ext import commands

import heavynode


class CustomBot(commands.Bot):
    def __init__(self, *args, **kwargs):
        self.cleanup = []
        super().__init__(*args, **kwargs)

    def add_cleanup(self, callback):
        self.cleanup.append(callback)

    async def close(self):
        for callback in self.cleanup:
            r = callback()
            # coroutines etc. return an awaitable object
            if inspect.isawaitable(r):
                await r
        await super().close()


logging.basicConfig()

server_id = 'd030737c'
hn = heavynode.Client(os.environ['heavynode_token'])

bot = CustomBot(command_prefix='!')
bot.add_cleanup(hn.shutdown)


@bot.command()
@commands.has_any_role('Admin', 'Mod')
async def add(ctx, player):
    """Add a player to the server whitelist. Must use exact Minecraft name."""
    await hn.send_command(server_id, f'whitelist add {player}')
    await ctx.send(f'"{player}" added to whitelist.')


@bot.command()
@commands.has_any_role('Admin', 'Mod')
async def remove(ctx, player):
    """Remove a player from the server whitelist. Must use exact Minecraft name."""
    await hn.send_command(server_id, f'whitelist remove {player}')
    await ctx.send(f'"{player}" removed from whitelist.')


@add.error
@remove.error
async def whitelist_error(ctx, error):
    if isinstance(error, commands.CheckFailure):
        await ctx.send('You must be a server admin to use this command.')
    else:
        raise error


bot.run(os.environ['discord_token'], reconnect=True)
